#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a GLib plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;;; Main Loop Hack
;;; package: (glib main)

(C-include "glib")

(define (glib-start)
  ;; Called from glib/make.scm, from a (load-option 'Glib).
  (if (not (plugin-available? "glib"))
      (error "GLib plugin not found"))
  (if (fix:zero? (with-glib-lock
		  (lambda ()
		    (C-call "start_glib"))))
      (error "Could not start Glib main loop."))
  (create-glib-thread))

(define-integrable (run-glib select-registry-handle time)
  (assert-glib-locked 'run-glib)
  (C-call "run_glib" select-registry-handle time)
  (assert-glib-locked 'run-glib-continue))

(define (maybe-yield-glib)
  ;; Used by callbacks that may have made threads runnable.
  (assert-glib-locked 'maybe-yield-glib)
  (if (other-running-threads?)
      (C-call "yield_glib")))

(define (stop-glib)
  ;; Sortof does the opposite of glib-start.
  (with-glib-lock
   (lambda ()
     (exit-glib-thread)
     (C-call "stop_glib"))))

(define (glib-select-trace?)
  (with-glib-lock
   (lambda ()
     (C-call "glib_select_trace_p"))))

(define (glib-select-trace! on?)
  (with-glib-lock
   (lambda ()
     (C-call "glib_select_trace" (if on? 1 0)))))