#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a GLib plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;;; Test procedures for the glib interface.

;;; GIO tests.

(define (gcp src dst)
  (let ((gsrc (open-input-gfile src))
	(gdst (open-output-gfile dst)))
    (let loop ()
      (let ((line (read-line gsrc)))
	(if (eof-object? line)
	    (begin
	      ;; Close the streams OR NOT, e.g. to test GCing of
	      ;; abandoned (quiet) ports.  Testing GCing of a port
	      ;; with an operation pending would be... useful, and
	      ;; tricky.
	      (close-input-port gsrc)
	      (close-output-port gdst))
	    (begin
	      (write-string line gdst) (newline gdst)
	      (loop)))))))

(define (gcat uri)
  (let ((gstream (open-input-gfile uri)))
    (let loop ()
      (let ((line (read-line gstream)))
	(if (eof-object? line)
	    (begin
	      ;; Close the gstream OR NOT, e.g. to test GCing of
	      ;; abandoned (quiet) ports.  Testing GCing of a port
	      ;; with an operation pending would be... useful, and
	      ;; tricky.
	      (close-input-port gstream))
	    (begin
	      (write-string line) (newline)
	      (loop)))))))

(define (ls pathname)
  (let ((names (map file-namestring
		    (directory-read (->simple-namestring pathname)))))
    (sort (delete! ".." (delete! "." names)) string<?)))

(define (gls uri)
  (sort (gdirectory-read uri) string<?))

(define ->simple-namestring
  (access ->simple-namestring (->environment '(gio))))

(define (assert-clean-ffi test-name)
  ;;(gc-flip)
  ;;(sleep-current-thread 100)		;Ensure GC deamons run? Hope they don't?
  (let ((ffi (->environment '(runtime ffi))))
    (if (not (zero? (car ((access registered-callback-count ffi)))))
	(error (string-append test-name" did not clean up its callbacks")))
    (if (not (null? (access malloced-aliens ffi)))
	(error (string-append test-name" did not free allocated memory:")
	       (map (lambda (elt)
		      (let ((alien (weak-car elt)))
			(if (eq? 'uchar (alien/ctype alien))
			    (c-peek-cstring alien)
			    alien)))
		    (access malloced-aliens ffi))))))